%DESCRIPTION:

%ARGUMENTS:
%file:    A csv file containing mass, volume, density, and porosity of volcanic deposits. The csv file  
%sep:     The columns can be separated with commas (","), semicolons (";") or spaces (""). The default field separator is commas.
%header:  The default value is TRUE, this means that each column has its own header. 
%runs:    It is the number of random runs that will be performed to compute the sample stability when the average density is weighted by volume.  
%         The default value is 1000
%probs:   The random runs are divide in quantiles. The probs vector gives the percentages. 
%pq:      Quantile of 95% (0.95) for the random runs.
%pev:     Closest sample size for errors of 1% and 5% (0.01,0.05) 
%breaks:  Intervals used to compute the histograms. The default vector spans from 0% to 100% of porosity in steps of 5%.
%minmaxv: If this argument is TRUE the plotting limits in the X-axis are given  
%         by the minimum and maximum porosity values of the field data. 
%plots:   The default value is TRUE. All the plots are shown in the screen.
%pdfs:    The default value is TRUE. All the plots are printed into pdf files.   
%txt:     The default value is TRUE. A txt file is written into the disk with the results.

%Example for running: 1.- results=stats("fielddata.csv")

%REFERENCES:
%Folk, RL, Ward, WC (1957) Brazos river bar: a study of the significance of grainsize parameters. J Sediment Petrol 27:3–26.
%Inman, DL (1952) Measures for describing the size distribution of sediments. J Sediment Petrol 22:125–145.

%FURTHER INFORMATION AND TROUBLESHOOTING:
% hugoortiz at boisestate.edu

function [ st ] = stats(file,sep,header,runs,probs,pq,pev,breaks,minmaxv,plots,pdfs,txt)
%Default input parameters
if ~exist('sep')    || isempty(sep)    ;  sep=','                     ;end
if ~exist('header') || isempty(header) ;  header=1                    ;end
if ~exist('runs')   || isempty(runs)   ;  runs=1000                   ;end
if ~exist('probs')  || isempty(probs)  ;  probs=(0:5:100)/100         ;end
if ~exist('pq')     || isempty(pq)     ;  pq=0.95                     ;end
if ~exist('pev')    || isempty(pev)    ;  pev=[0.01 0.05]             ;end
if ~exist('breaks') || isempty(breaks) ;  breaks=0:0.05:1             ;end
if ~exist('minmaxv')|| isempty(minmaxv);  minmaxv=1                   ;end
if ~exist('plots')  || isempty(plots)  ;  plots=1                     ;end
if ~exist('pdfs')   || isempty(pdfs)   ;  pdfs=1                      ;end
if ~exist('txt')    || isempty(txt)    ;  txt=1                       ;end

%Read csv file
%a=readtable(file,'Delimiter',sep,'ReadVariableNames',header);
a=importdata(file,sep,header);
if header == 1
    a=a.data;
end
%Find min and max values of mass, volume, density (rho) and porosity (phi)
m=[min(a(:,1)) max(a(:,1))];v=[min(a(:,2)) max(a(:,2))];rho=[min(a(:,3)) max(a(:,3))];phi=[min(a(:,4)) max(a(:,4))];

%Computing the total mass and volume.
tm=sum(a(:,1));tv=sum(a(:,2));

%Number of samples in the file
n=size(a);n=n(1);

%Standard deviations of density and porosity
srho=std(a(:,3));sphi=std(a(:,4));

%Frequential averages of mass, volume, density and porosity
fam=mean(a(:,1));fav=mean(a(:,2));farho=mean(a(:,3));faphi=mean(a(:,4));

%Weighting vector: volume
wv=a(:,2)/sum(a(:,2));

%Density average weighted by volume
wvarho=sum(wv.*a(:,3));

%Porosity average weighted by volume
wvaphi=sum(wv.*a(:,4));

%Graphical statistics for density: Inman (1952)
grho=quantile(a(:,3),[0.05 0.15 0.16 0.25 0.5 0.75 0.84 0.95]);
gmedianrho=grho(5); %graphical median
gsigmarho=(grho(7)-grho(3))/2; %graphical standard deviation
gskrho=(grho(7)+grho(3)-2*grho(5))/(2*(grho(7)-grho(3))); %graphical skewness

%Graphical statistics for density: Folk and Ward (1957) 
gmeanrho=(grho(3)+grho(5)+grho(7))/3; %graphical mean
gsigmairho=(grho(7)-grho(3))/4 + (grho(8)-grho(1))/6.6; %inclusive standard deviation
gksirho=(grho(7)+grho(3)-2*grho(5))/(2*(grho(7)-grho(3))) + (grho(8)+grho(1)-2*grho(5))/(2*(grho(8)-grho(1))); %inclusive skewness
gkgrho=(grho(8)-grho(1))/(2.44*(grho(6)-grho(4))); %graphical kurtosis

%Graphical statistics for porosity: Inman (1952)
gphi=quantile(a(:,4),[0.05 0.15 0.16 0.25 0.5 0.75 0.84 0.95]);
gmedianphi=gphi(5); %graphical median
gsigmaphi=(gphi(7)-gphi(3))/2; %graphical standard deviation
gskphi=(gphi(7)+gphi(3)-2*gphi(5))/(2*(gphi(7)-gphi(3)));  %graphical skewness

%Graphical statistics for porosity: Folk and Ward (1957)
gmeanphi=(gphi(3)+gphi(5)+gphi(7))/3; %graphical mean
gsigmaiphi=(gphi(7)-gphi(3))/4 + (gphi(8)-gphi(1))/6.6; %inclusive standard deviation
gksiphi=(gphi(7)+gphi(3)-2*gphi(5))/(2*(gphi(7)-grho(3))) + (gphi(8)+gphi(1)-2*gphi(5))/(2*(gphi(8)-gphi(1))); %inclusive skewness
gkgphi= (gphi(8)-gphi(1))/(2.44*(gphi(6)-gphi(4))); %graphical kurtosis

%To compute porosity histograms. Histogram spacing is based on breaks vector
phih=[];
vh=[];
labels=[];

for k = 1:(length(breaks)-1)
    
    id=find( a(:,4)>=breaks(k) & a(:,4)<breaks(k+1) == 1 );
    phih=[phih length(id)];
    vh=[vh sum(wv(id))];
    labels{k}=[num2str(100*breaks(k)) '-' num2str(100*breaks(k+1))];
    if(phi(1)>=breaks(k) & phi(1)<breaks(k+1)); il=k; end %min value in the histogram plot
    if(phi(2)>=breaks(k) & phi(2)<breaks(k+1)); sl=k; end %max value in the histogram plot
    
end

phih=phih/n;

%To compute density histograms.
rhoh=[];
lrho=[];
brho=(fix(min(a(:,3))/100)*100):100:((fix(max(a(:,3))/100)+1)*100);

for k = 1:(length(brho)-1)
    
    id=find( a(:,3)>=brho(k) & a(:,3)<brho(k+1) == 1 );
    rhoh=[rhoh sum(wv(id))];
    lrho{k}=[num2str(brho(k)/1000) '-' num2str(brho(k+1)/1000)];
end

%The matrix containing all the histogram values is define
H=[vh;phih];
%If the plot parameter minmax is defined, the condition is applied
if(minmaxv == 1);H=H(:,il:sl);labels=labels(il:sl); end
%Random runs of sample for computing the absolute error of the volume and density.
V=zeros(runs,n);
RHO=zeros(runs,n);
for j = 1:runs
    ij=randsample(n,n);
    vj=a(ij,2);
    rhoj=a(ij,3);
    vij=[];
    rhoij=[];
    for i = 1:n
        wvij=vj(1:i)/sum(vj(1:i));
        vi=sum(wvij.*rhoj(1:i));vij=[vij abs(vi-wvarho)./wvarho];
        rhoi=mean(rhoj(1:i));rhoij=[rhoij abs(rhoi-farho)./farho];
    end
    V(j,:)=vij;
    RHO(j,:)=rhoij;
end

%Computing quantile matrices for mass, volume and density random runs.
QV=zeros(length(probs),n);
QRHO=zeros(length(probs),n);

for w = 1:n
    QV(:,w)=quantile(V(:,w),probs);
    QRHO(:,w)=quantile(RHO(:,w),probs);
end

%Finding the n values closest to the absolute errors defined in the "pev" vector.
qi=find(probs == pq);
nwmrho=[];
nwvrho=[];
for s = 1:length(pev)
    dwvrho=abs(QV(qi,:)-pev(s));
    nwvrhos=find(dwvrho == min(dwvrho));
    nwvrho=[nwvrho nwvrhos];
end
%Putting together all the n values related with each "pe" entry
N=nwvrho;
%Slope, considering the absolute error percentages provided in the "pe" vector
sswvrho=[];
for alpha = 1:length(pev)
    sswvalpha=QV(qi,nwvrho(alpha))*100/(n-nwvrho(alpha));
    sswvrho=[sswvrho sswvalpha];
end

%Prefix name
[pathstr,fn,ext] = fileparts(file);

%Plotting section

%Histograms plot of  volume and density

    function plotbc()
        pH=bar(100*H');
        set(pH(1),'FaceColor',[0 0 0])
        set(pH(2),'FaceColor',[0.5 0.5 0.5])
        title(['Abundance Histogram Comparison: Porosity; n= ' num2str(n)])
        xlabel('Porosity (%)')
        ylabel('Abundance (%)')
        legend('Weighted by volume','Frequency')
        set(gca, 'XTickLabel', labels)
    end

%Plotting of the random runs for the "pq" quantile

    function plotre()
        plot(QV(qi,:)*100,'-ko','MarkerFaceColor','k')
        line([0 length(QV(qi,:))*1.2],[1 1],'LineWidth',2,'LineStyle','-.','Color',[0 0.6 0]);
        line([0 length(QV(qi,:))*1.2],[5 5],'LineWidth',2,'LineStyle','-.','Color',[0 0.6 0]);
        text(3,2.5,'1%','Color',[0 0.6 0])
        text(3,6.5,'5%','Color',[0 0.6 0])
        xlim([0 length(QV(qi,:))*1.1]);
        title('Stability Curve')
        xlabel('Sample Size')
        ylabel('2-Sigma Absolute Error (%)')
        annotation('textbox',[0.7 0.8 0.1 0.1],'String',...
            {fn,['n: ' num2str(n)],['runs= ' num2str(runs)]})
    end

%Cumulative plots and histograms for density and porosity

    function ploter()
        subplot(2,2,1)
        bar(100*rhoh,'FaceColor',[0.5 0.5 0.5])
        xlabel('Density (kg/m^3)')
        ylabel('Abundance (vol. %)')
        title('Density Histogram')
        %set(gca, 'XTickLabel', lrho,'FontSize',1)
        [sa,ej]=sort(a(:,3));
        mina=min(a(:,3));
        maxa=max(a(:,3));
        subplot(2,2,2)
        plot(a(ej,3),100*cumsum(wv(ej)),'-ok')
        line([mina maxa],[5   5],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[16 16],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[25 25],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[50 50],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[75 75],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[84 84],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[95 95],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        xlim([mina maxa])
        xlabel('Density (kg/m^3)')
        ylabel('Cumulative Abundance (vol. %)')
        title('Cumulative Density Plot')
        
        subplot(2,2,3)
        bar(100*H(1,:),'FaceColor',[0.5 0.5 0.5])
        xlabel('Porosity (%)')
        ylabel('Abundance (vol. %)')
        title('Porosity Histogram')
        %set(gca, 'XTickLabel', labels)
        
        subplot(2,2,4)
        [sa,ej]=sort(a(:,4));
        mina=100*min(a(:,4));
        maxa=100*max(a(:,4));
        plot(100*a(ej,4),100*cumsum(wv(ej)),'-ok')
        line([mina maxa],[5 5],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[16 16],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[25 25],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[50 50],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[75 75],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[84 84],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        line([mina maxa],[95 95],'LineWidth',1.5,'LineStyle','--','Color',[0 0.6 0]);
        xlim([mina maxa])
        xlabel('Porosity (%)')
        ylabel('Cumulative Abundance (vol. %)')
        title('Cumulative Density Plot')
        
    end

%Bringing the plots into the screen
if (plots ==1)
    figure(1); plotbc(); figure(2); plotre(); figure(3); ploter();
end

%Printing the plots to pdf
if (pdfs ==1 & plots == 1)
    print('-f1',strcat(fn,'-bc'),'-dpdf');
    print('-f2',strcat(fn,'-re'),'-dpdf');
    ploter(); print('-f3',strcat(fn,'-er'),'-dpdf');
elseif (pdfs ==1 & plots == 0)
    figure(1); plotbc(); print('-f1',strcat(fn,'-bc'),'-dpdf');
    figure(2); plotre(); print('-f2',strcat(fn,'-re'),'-dpdf');
    figure(2); ploter(); print('-f3',strcat(fn,'-er'),'-dpdf');
end

%Writing the results to a txt file

if (txt == 1 )
    
    table=cell(38,2);
    table(:,1)={'item', 'n','minm','maxm','fam','tm','minv','maxv','fav',...
        'tv','minrho','maxrho','stdrho','farho','wvarho','minphi','maxphi',...
        'stdphi','faphi','wvaphi','nwvrho_1%','nwvrho_5%','sswvrho_%1',...
        'sswvrho_%5','gmedianrho','gsigmarho','gskrho','gmeanrho','gsigmairho',...
        'gksirho','gkgrho','gmedianphi','gsigmaphi','gskphi','gmeanphi',...
        'gsigmaiphi','gksiphi','gkgphi'};
    table(1,2)={'value'};
    table(2:end,2)={n,m(1),m(2),fam,tm,v(1),v(2),fav,tv,rho(1),rho(2),...
        srho,farho,wvarho,phi(1),phi(2),sphi,faphi,wvaphi,nwvrho(1),...
        nwvrho(2),sswvrho(1),sswvrho(2),gmedianrho,gsigmarho,gskrho,...
        gmeanrho,gsigmairho,gksirho,gkgrho,gmedianphi,gsigmaphi,gskphi,...
        gmeanphi,gsigmaiphi,gksiphi,gkgphi};
    
    fileID = fopen(strcat(fn,'-M.txt'),'w');
    formatSpec = '%s %s\n';
    fprintf(fileID,formatSpec,table{1,:});
    formatSpec = '%s %f\n';
    for k=2:length(table(:,1))
        fprintf(fileID,formatSpec,table{k,:});
    end
    fclose(fileID);
end

%All the computed parameters are stored in the metacharacter "st"
%Defining "st" as a structure
%Setting up all the result parameters in "st"

st=struct('n',n,'minm',m(1),'maxm',m(2),'fam',fam,'tm',tm,'minv',v(1),'maxv',v(2),'fav',fav,'tv',tv,...
    'minrho',rho(1),'maxrho',rho(2),'stdrho',srho,'farho',farho,'wvarho',wvarho,'minphi',phi(1),...
    'maxphi',phi(2),'stdphi',sphi,'faphi',faphi,'wvaphi',wvaphi,'nwvrho',nwvrho,'sswvrho',sswvrho,...
    'H',H,'wv',wv,'phi',a(:,4),'gmedianrho',gmedianrho,...
    'gsigmarho', gsigmarho,'gskrho',gskrho,'gmeanrho',gmeanrho,'gsigmairho',gsigmairho,'gksirho',...
    gksirho,'gkgrho',gkgrho,'gmedianphi',gmedianphi,'gsigmaphi',gsigmaphi,'gskphi',gskphi,...
    'gmeanphi',gmeanphi,'gsigmaiphi',gsigmaiphi,'gksiphi',gksiphi,'gkgphi',gkgphi);

end


